#ifndef ERRORH
#define ERRORH
 
/** 
   \file error.h

   \brief Error and warning functions.

   A module that defines different error and warning functions used
   in the CuikSuite applications.<br>
   To have a unique error function helps when debugging.
   
   \sa error.c, error_world.h.
*/

/** 
   \brief General error function.

   General error function.

   \param s A string describing the error.
*/
void Error(const char *s);

/** 
   \brief Syntax errors in \e .loops files.

   This function is called by the \e .loops file parser when encountering 
   syntax errors.

   \param s A string generated by Bison (Normally it contains "Syntax Error").

   \return EXIT_FAILURE.
*/
int Readloopserror(const char *s);

/** 
   \brief Syntax errors in \e .cuik files.

   This function is called by the \e .cuik file parser when encountering 
   syntax errors.

   \param s A string generated by Bison (Normally it contains "Syntax Error").

   \return EXIT_FAILURE.
*/
int Readcuiksystemerror(const char *s); 

/** 
   \brief Syntax errors in \e .param files.

   This function is called by the \e .param file parser when 
   encountering syntax errors.

   \param s A string generated by Bison (Normally it contains "Syntax Error").

   \return EXIT_FAILURE.
*/
int ReadParameterserror(const char *s); 

/** 
   \brief Semantic errors in \e .loops files.

   This function is called by the \e .loops file parser when it 
   encountrs a semantic error.

   \param s A string describing the error.

   \return EXIT_FAILURE.
*/
void ReadloopsSemanticError(const char *s);

/** 
   \brief Semantic errors in \e .cuik files.

   This function is called by the \e .cuik file parser when it 
   encountrs a semantic error.

   \param s A string describing the error.

   \return EXIT_FAILURE.
*/
void ReadcuiksystemSemanticError(const char *s);

/** 
   \brief General warning function.

   General warning function.

   \param s A string describing the warning.
*/
void Warning(const char *s);

#endif
