
#include "cuiksystem.h"
#include "parameters.h"

#include "defines.h"
#include "error.h"
#include "filename.h"

#include <string.h>
#include <stdlib.h>


/** \file cuikangles.c
    \brief Main body of the \ref cuikangles.c "cuikcluter" application.
 
    <center>
    \dot
    digraph applications {
      size="7,15"
      node [shape=record, style=filled, fontname=Helvetica, fontsize=12];

      cluster    [label="cluster"    color="#99ff99" shape=ellipse ];
      
      kin     [label="cuik"   color="#ff9999"];
      sol     [label="sol"     color="#ff9999"];

      sol -> cuik
      kin -> cluster
    }
    \enddot
    </center>

    This application cluster lists of boxes. 
    <br>

    \sa cuiksystem.h cuiksystem.c
*/

/** 
   \brief Main body of the \b cuikangles application.

   Main body of the \b cuikangles application.

   Use:
      - \b cuikangles \e problem_name.sol
      .
   where
      - \e problem_name contains the solutions to cluster
      .

   The \e .sol extension is optional.
   
   \param argc Number of elements in the command line that starts the application (i.e., the
               \b cuik command itself and its arguments).
   \param arg Strings with the arguments.
   
   \return EXIT_SUCCESS (0) if the execution worked out fine and EXIT_FAILURE if not.
*/
int main(int argc, char **arg)
{
  FILE *f_out;             /* File where the solutions are printed as they 
			      are found */
 
  FILE *f_in;            /* Used to read-and-print the equation input file in the output file */
  
  Tlist solutions;

  Tfilename fsols;
  Tfilename fangles;

  if (argc>1)
    {      
      /* Read the list of boxes */
      CreateFileName(NULL,arg[1],NULL,SOL_EXT,&fsols);
      f_in=fopen(GetFileFullName(&fsols),"r");
      if (!f_in)
	Error("Can not open the file with the solutions");

      ReadCuikBoxList(f_in,&solutions);
      
      /* Save the list of boxes */
      CreateFileName(NULL,arg[1],"_angles",SOL_EXT,&fangles);
      f_out=fopen(GetFileFullName(&fangles),"w");
      if (!f_out)
	Error("Can not open the file for the angles");
      SaveCuikBoxList(f_out,1,TRUE,FALSE,FALSE,FALSE,NULL,&solutions);

      /* Release memory */
      DeleteCuikBoxList(&solutions);
      
      DeleteFileName(&fsols);
      DeleteFileName(&fangles);
    }
  else
    {
      fprintf(stderr,"  Wrong number of parameters.\n");
      fprintf(stderr,"  Use:\n");   
      fprintf(stderr,"      cuikangles <problem file name>.sol \n");
      fprintf(stderr,"  where <problem file name> contains the solutions\n");
      fprintf(stderr,"    (the '.sol' extension is not required)\n");
    }

  return(EXIT_SUCCESS);
}

