
#include "cuiksystem.h"
#include "parameters.h"

#include "defines.h"
#include "error.h"
#include "filename.h"
#include "random.h"

#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>

/** \file cuikloops2cuik.c
    \brief Main body of the \ref cuikloops2cuik.c "cuikloops2cuik" application.
 
    <center>
    \dot
    digraph applications {
      size="7,15"
      node [shape=record, style=filled, fontname=Helvetica, fontsize=12];

      cuik    [label="cuik"    color="#99ff99" shape=ellipse ];
      
      param   [label="param"   color="#ff9999"];
      kin     [label="cuik"   color="#ff9999"];
      maf     [label="maf"     color="#ff9999"];

      param -> cuik
      kin -> cuik
      cuik -> maf
    }
    \enddot
    </center>

    Generates the MAF (Multi-affine) file from the file of equations.

    The solver only works on MAF files. Equation (cuik) files are only
    provided by convinience and they are alwasy translated into a MAF
    file 

    In complex problems, the MAF file is much more compact than the
    eauations themselves.
    <br>

    \sa cuiksystem.h cuiksystem.c
*/

/** 
   \brief Main body of the \b cuikloops2cuik application.

   Main body of the \b cuikloops2cuik application.

   Use:
      - \b cuikloops2cuik \e problem_name.cuik
      .
   where
      - \e problem_name contains the kinematic equations
      .

   The \e .cuik extension is optional.
   
   \param argc Number of elements in the command line that starts the application (i.e., the
               \b cuik command itself and its arguments).
   \param arg Strings with the arguments.
   
   \return EXIT_SUCCESS (0) if the execution worked out fine and EXIT_FAILURE if not.
*/
int main(int argc, char **arg)
{
  Tfilename fparam;
  Tparameters parameters;  /* Parameters used in the Cuik process */
  
  if (argc>1)
    {
      /*Init parameters*/
      CreateFileName(NULL,arg[1],NULL,PARAM_EXT,&fparam);
      InitParametersFromFile(GetFileFullName(&fparam),&parameters);
      
      Loops2Cuik(&parameters,arg[1]);
      
      DeleteParameters(&parameters);
      DeleteFileName(&fparam);
    }
  else
    {
      fprintf(stderr,"  Wrong number of parameters.\n");
      fprintf(stderr,"  Use:\n");   
      fprintf(stderr,"      cuikloops2cuik <problem file name>.loops \n");
      fprintf(stderr,"  where <problem file name> contains the kinematic equations\n");
      fprintf(stderr,"    (the '.cuik' extension is not required)\n");
    }

  return(EXIT_SUCCESS);
}

