#include "equations.h"

#include "defines.h"
#include "error.h"
#include "geom.h"

#include <math.h>

/** \file equations.c
    \brief Implementation of the functions operating on sets of Tequations.

    Implementation of the functions operating on sets of Tequations.
 
   \sa Tequations, equations.h.
*/



/**********************************************************************************/
/**********************************************************************************/
/**********************************************************************************/

void InitEquations(Tequations *eqs)
{
  eqs->neq=0; /* Total equations */

  eqs->m=INIT_NUM_EQUATIONS;
  NEW(eqs->equation,eqs->m,Tequation*);
}

void CopyEquations(Tequations *eqs_dst,Tequations *eqs_src)
{
  unsigned int i;

  eqs_dst->neq=eqs_src->neq;

  eqs_dst->m=eqs_src->m;
  NEW(eqs_dst->equation,eqs_dst->m,Tequation*);
  for(i=0;i<eqs_src->neq;i++)
    {
      NEW(eqs_dst->equation[i],1,Tequation);
      CopyEquation(eqs_dst->equation[i],eqs_src->equation[i]);
    }
}

unsigned int NEquations(Tequations *eqs)
{
  return(eqs->neq);
}

void AddEquation(Tequation *equation,Tequations *eqs)
{
  if (eqs->neq==eqs->m)
    {
      MEM_DUP(eqs->equation,eqs->m,Tequation *);
    }
  
  NEW(eqs->equation[eqs->neq],1,Tequation);
  CopyEquation(eqs->equation[eqs->neq],equation);
  eqs->neq++;
}

Tequation *GetEquation(unsigned int n,Tequations *eqs)
{
  #if (_DEBUG>0)
  if (n>=eqs->neq)
    Error("Index out of range in GetEquation");
  #endif
  return(eqs->equation[n]);
}

void EvaluateEqualityEquations(double *v,double *r,Tequations *eqs)
{
  unsigned int i,k;

  k=0;
  for(i=0;i<eqs->neq;i++)
    {
      if (GetEquationCmp(eqs->equation[i])==EQU)
	{
	  r[k]=EvaluateEquation(v,eqs->equation[i]);
	  k++;
	}
    }
}

void PrintEquations(FILE *f,char **varNames,Tequations *eqs)
{
  unsigned int i;
  
  if (eqs->neq>0)
    {
      fprintf(f,"\n[EQS]\n");
      for(i=0;i<eqs->neq;i++)
	{
	  fprintf(f,"   ");
	  PrintEquation(f,varNames,eqs->equation[i]);
	}
    }
}

void DeleteEquations(Tequations *eqs)
{
  unsigned int i;

  for(i=0;i<eqs->neq;i++)
    { 	 
      DeleteEquation(eqs->equation[i]);
      free(eqs->equation[i]);
    }
  free(eqs->equation);
}
