%{ 
/*
* Lexical analizer for the function that initilizes a set of equations from a file
*/

/** \cond */


#include "readloopstypes.h"
#include "readloops.tab.h"

#include "defines.h" 
#include "error.h"

#include <stdlib.h> 
#include <string.h>


int Readcuiksystemlex(void);

extern unsigned int RLPline;

%}
%s IN_COMMENT

EMPTY                           [ \t]+ 
RETURN                          [\n|\r\n]
DIGIT                           [0-9] 
LETTER                          [a-zA-Z] 
R_NUMBER                        (({DIGIT}+(\.{DIGIT}+)?)|(\.{DIGIT}+))((E|e)[+-]?{DIGIT}+)? 
I_NUMBER                        {DIGIT}+
IDENTIFIER                      {LETTER}({DIGIT}|{LETTER}|"_")* 
COMMENT                         \%[^{\n|\r\n|\{}]*
COMMENT2                        \#[^{\n|\r\n|\{}]*
COMMENT3                        \/\/[^{\n|\r\n|\{}]*
 
%%
{COMMENT}                       { }  
{COMMENT2}                      { }   
{COMMENT3}                      { }   

<INITIAL>{
  "/*"              BEGIN(IN_COMMENT);
}

<IN_COMMENT>{
  "*/"      BEGIN(INITIAL);
  [^*\n]+   // eat comment in chunks
  "*"       // eat the lone star
  \n        RLPline++;
}

{EMPTY}                         { } 

{RETURN}                        { RLPline++; } 


"[CONSTANTS]"                   {
                                  return(_CONSTANTS);
                                }

"[SYSTEM VARS]"                 {
                                  return(_VARS);
                                }

"[LOOPS]"                       {
                                  return(_LOOPS);
                                }

":="                            {
                                  return(_ASSIGN);
                                }

PI                              {
                                  return(_PI);
                                }

SIN                             {
                                  return(_SIN);
                                }

COS                             {
                                  return(_COS);
                                }

TAN                             {
                                  return(_TAN);
                                }

COT                             {
                                  return(_COT);
                                }

ACOS                            {
                                  return(_ACOS);
                                }

ASIN                            {
                                  return(_ASIN);
                                }

ATAN                            {
                                  return(_ATAN);
                                }

ATAN2                           {
                                  return(_ATAN2);
                                }

EXP                             {
                                  return(_EXP);
                                }

SQRT                            {
                                  return(_SQRT);
                                }

ABS                             {
                                  return(_ABS);
                                }

"="                             {
                                  return(_EQU); 
                                }

"ID"                            {
                                  return(_ID); 
                                }

"Rx"                            {
                                  return(_RX); 
                                }

"Ry"                            {
                                  return(_RY); 
                                }

"Rz"                            {
                                  return(_RZ); 
                                }

"Tx"                            {
                                  return(_TX); 
                                }

"Ty"                            {
                                  return(_TY); 
                                }

"Tz"                            {
                                  return(_TZ); 
                                }

"Trm"                           {
                                  return(_TRM); 
                                }

"Trq"                           {
                                  return(_TRQ); 
                                }

"DHR"                           {
                                  return(_DHR); 
                                }

"^-1"                           {
                                  return(_INV); 
                                }



{I_NUMBER}                      { 
				  char string_tmp[100];

				  if (Readloopsleng>100)
                                    Readloopserror("Integer numbers can not have more than 100 digits");

				  memcpy(string_tmp,Readloopstext,(unsigned int)Readloopsleng);
				  string_tmp[Readloopsleng]=0;
				  Readloopslval.int_number=atoi(string_tmp);
				  return(_INTEGER); 
				} 
 
{R_NUMBER}                      { 
				  char string_tmp[100];
				  
				  if (Readloopsleng>100)
                                    Readloopserror("Real numbers can not have more than 100 digits");

				  memcpy(string_tmp,Readloopstext,(unsigned int)Readloopsleng);
				  string_tmp[Readloopsleng]=0;
				  Readloopslval.real_number=atof(string_tmp);
				  
				  return(_REAL);
				} 
  
{IDENTIFIER}                    { 
                                  if (Readloopsleng>100)
                                    Readloopserror("Identifiers can not have more than 100 characters");

                                  NEW(Readloopslval.id,Readloopsleng+1,char);
 				  memcpy(Readloopslval.id,Readloopstext,(unsigned int)Readloopsleng);
  
 				  Readloopslval.id[Readloopsleng]=0;

				  return(_IDENTIFIER);  
				} 

\{                              {
                                   Readloopserror("The input file can not include the symbol '{'");
                                }

.                               { return(Readloopstext[0]); }

%%
/** \endcond */

/**
  \brief Function called at the end of the parsing process.

  This function is automatically called by the parser at the end of a file.
  If 1 is returned the lexical analysis is stopped.
 
  \return 1 to stop the parsing. 
*/
int Readloopswrap()
{
  return(1);
}
   
