%{ 
/** \cond */
#include "readloopstypes.h"

#include "cuiksystem.h"
#include "boolean.h"
#include "error.h"
#include "constants.h"
#include "loops.h"
#include "variables.h"
#include "defines.h"
#include "parameters.h"
#include "filename.h"

#include <stdlib.h> 
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <time.h>
  /*Lex and Yacc variables*/
  extern FILE *Readloopsin; 

  /*Lex and Yacc functions*/
  int Readloopslex(void); 

  /*Our own variables*/
  extern unsigned int RLPline; /*line number currently processed (incremented by the LEX processor)*/
  
  Tparameters *rlp_parameters;

  /*Global pointer to allow the different parts of the parser to acces the equations being initialized*/
  Tconstants rlp_constants;
  Tloops rlp_loops;
  Tvariables rlp_variables;
 
%}

%union 
{
  char *id; 
  int int_number;
  double real_number;
  boolean flag;
  TexprLoops expr;
  Ttransform transform;
  Ttransforms transforms;
} 
 
%start problem

%token  _CONSTANTS _VARS _LOOPS _ASSIGN _PI _SIN _COS _TAN _COT _ACOS _ASIN _ATAN _ATAN2 _EXP _SQRT _ABS _EQU _ID _RX _RY _RZ _TX _TY _TZ _TRM _TRQ _DHR _INV

%token <id> _IDENTIFIER
%token <int_number> _INTEGER
%token <real_number> _REAL

%type <expr> expr
%type <int_number> var_type
%type <int_number> transform_type
%type <flag> opt_inv
%type <transform> transform 
%type <transforms> transforms 
%type <transforms> more_transforms 


%left _MIN_PRECEDENCE
%left '+' '-' 
%left '*' '/'
%left '^'
%left '\''
%right _MAX_PRECEDENCE 
 
%% 

problem: ct_defs var_defs loops 
       ;

ct_defs : _CONSTANTS ct_list  
        |
        ;
         
ct_list : ct_definition ct_list
        |
        ;

ct_definition: _IDENTIFIER _ASSIGN expr
               {
		 if (GetConstantWithName($1,&rlp_constants)!=NO_UINT)
		   {
		     char s[250];
		     
		     sprintf(s,"Duplicated constant %.150s",$1);
		     ReadloopsSemanticError(s);
		   }

		 if ($3.id!=NULL)
		   ReadloopsSemanticError("Constant assigned to a variable");

	         AddConstant($1,$3.value,&rlp_constants);
		 free($1);
               }
             ;


expr : '+' expr    %prec _MAX_PRECEDENCE
         {
	   if ($2.id!=NULL)
	     ReadloopsSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$2.value;
         }
        | '-' expr    %prec _MAX_PRECEDENCE
         {
	   if ($2.id!=NULL)
	     ReadloopsSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=-$2.value;
         }
        | expr '+' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadloopsSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$1.value+$3.value;
         }
        | expr '-' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadloopsSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$1.value-$3.value;
         }
        | expr '*' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadloopsSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$1.value*$3.value;
         }
        | expr '^' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadloopsSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=pow($1.value,$3.value);
         }
        | expr '/' expr 
         {
	   if (($1.id!=NULL)||($3.id!=NULL))
	     ReadloopsSemanticError("No operation is allowed with variables");

	   if ($3.value==0.0)
	     ReadloopsSemanticError("Division by zero");
	   
	   $$.id=NULL;
	   $$.value=$1.value/$3.value;
         }
        | '(' expr ')'
         {
	   if ($2.id!=NULL)
	     ReadloopsSemanticError("No operation is allowed with variables");

	   $$.id=NULL;
	   $$.value=$2.value;
         }
        | '(' expr '?' expr ':' expr ')'
          {
	    if (($2.id!=NULL)||($4.id!=NULL)||($6.id!=NULL))
	     ReadloopsSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    if ($2.value>0)
	      $$.value=$4.value;
	    else
	      $$.value=$6.value;
	  }
        | _PI 
          {
	    $$.id=NULL;
	    $$.value=M_PI;
	  }
        | _SIN '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadloopsSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=sin($3.value);
	  }
        | _COS '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadloopsSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=cos($3.value);
	  }
        | _TAN '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadloopsSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=tan($3.value);
	  }
        | _COT '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadloopsSemanticError("No operation is allowed with variables");
	   
	    $$.id=NULL;
	    $$.value=tan(M_PI_2-$3.value);
	  }
        | _ASIN '(' expr ')'
	  {
	    if ($3.id!=NULL)
	      ReadloopsSemanticError("No operation is allowed with variables");
	   
	    $$.id=NULL;
	    $$.value=asin($3.value);
	  }
        | _ACOS '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadloopsSemanticError("No operation is allowed with variables");
	   
	    $$.id=NULL;
	    $$.value=acos($3.value);
	  }
        | _ATAN '(' expr ')'
          {
	    if ($3.id!=NULL)
	      ReadloopsSemanticError("No operation is allowed with variables");
	   
	    $$.id=NULL;
	    $$.value=atan($3.value);
	  }
        | _ATAN2 '(' expr ',' expr ')'
          {
	    if (($3.id!=NULL)||($5.id!=NULL))
	      ReadloopsSemanticError("No operation is allowed with variables");
	    
	    $$.id=NULL;
	    $$.value=atan2($3.value,$5.value);
	  }
        | _EXP '(' expr ')'
         {
	   if ($3.id!=NULL)
	     ReadloopsSemanticError("No operation is allowed with variables");
	   
	   $$.id=NULL;
	   $$.value=exp($3.value);
	 }
        | _SQRT '(' expr ')'
         {
	   if ($3.id!=NULL)
	     ReadloopsSemanticError("No operation is allowed with variables");
	   
	   $$.id=NULL;
	   $$.value=sqrt($3.value);
	 }
        | _ABS '(' expr ')'
         {
	   if ($3.id!=NULL)
	     ReadloopsSemanticError("No operation is allowed with variables");
	   
	   $$.id=NULL;
	   $$.value=fabs($3.value);
	 }
        | _IDENTIFIER
         {
	   unsigned int nc;

	   nc=GetConstantWithName($1,&rlp_constants);

	   if (nc!=NO_UINT)
	     {
	       $$.id=NULL;
	       $$.value=GetConstantValue(nc,&rlp_constants);
	       free($1);
	     }
	   else
	     {
	       if (GetVariableID($1,&rlp_variables)==NO_UINT) 
		 {
		   char s[250];
		   
		   sprintf(s,"Undefined variable %.150s",$1);
		   ReadloopsSemanticError(s);
		 }
	       else
		 $$.id=$1;
	     }
	 }
        | _INTEGER
         {
	   $$.id=NULL;
	   $$.value=(double)$1;
         } 
        | _REAL
         {
	   $$.id=NULL;
	   $$.value=$1;
         }
        ; 

var_defs : _VARS
           range_list
         ;

range_list: range_definition range_list
          |
          ;

range_definition : _IDENTIFIER var_type '[' expr ',' expr ']'
                  {
		    Tinterval i_user; 
		    Tvariable v;
		    unsigned int nv;
		    
		    if (($4.id!=NULL)||($6.id!=NULL))
		      ReadloopsSemanticError("Can not define a range with variables");

		    if (GetConstantWithName($1,&rlp_constants)!=NO_UINT)  
		      {
			char s[250];
			
			sprintf(s,"Constant %.150s redefined as a variable",$1);
			ReadloopsSemanticError(s);
		      }

		    if (GetVariableID($1,&rlp_variables)!=NO_UINT) 
		      {
			char s[250];
			
			sprintf(s,"Repeated range declaration for variable %s",$1);
			ReadloopsSemanticError(s);
		      }		    
		    
		    NewInterval($4.value,$6.value,&i_user);
		    
		    if (EmptyInterval(&i_user))
		      ReadloopsSemanticError("Empty Interval");


		    NewVariable($1,$2,&v);
		    
		    SetVariableInterval(&i_user,&v);
		    
		    nv=AddVariable(&v,&rlp_variables);

		    DeleteVariable(&v);

		    /* Define the offset variable assocated with the rotation */
		    if (($2==VAR_ROT)&&($4.value==-M_PI)&&($6.value==M_PI))
		      {
			char *on;
			
			NEW(on,10,char);
			sprintf(on,"_o_%u",nv);
			
			NewOffsetVariable(on,nv,&v);
			AddVariable(&v,&rlp_variables);
			
			DeleteVariable(&v);
			free(on);
		      }
		    
		    free($1);
		  }
                 ;

var_type : ':'
           {
	     $$=VAR_TRANS;
	   }
         |
           '~'
           {
	     $$=VAR_ROT;
	   }
         ;

loops : _LOOPS
        loops_list
      |
      ;

loops_list : loop 
             loops_list
           |
           ;

loop: transforms _EQU transforms ';'
      {
	unsigned int rl;
	
	// Move all to the LHS -> $1 = $1 * $3^-1
	PostMultiplyTransforms(&($3),TRUE,COMPACT_TRANSFORMS,&($1));

	AddLoop(&($1),&rlp_loops);

	// If requested, we generate redundant equations by rotating the loop.
	// This is typicallu used in combination with simplification.
	rl=(unsigned int)GetParameter(CT_ROTATE_LOOPS,rlp_parameters);
	if (rl==2)
	  {
	    unsigned int i,nv;

	    nv=NTransformsVariables(&($1));

	    for(i=1;i<nv;i++)
	      {
		RotateTransforms(0,TRUE,&($1));
		AddLoop(&($1),&rlp_loops);
	      }
	  }
	else
	  {
	    if (rl==1)
	      {
		unsigned int i,nt;

		nt=NTransforms(&($1));

		for(i=1;i<nt;i++)
		  {
		    RotateTransforms(0,FALSE,&($1));
		    AddLoop(&($1),&rlp_loops);
		  }
	      }
	  }
	      
	DeleteTransforms(&($1));
	DeleteTransforms(&($3));
      }
    ;

transforms : transform more_transforms
             {
	       PreMultiplyTransforms(&($1),&($2),&($$));
	       
	       DeleteTransform(&($1));
	       DeleteTransforms(&($2));
	     }
           ;

more_transforms : '*' transform more_transforms  
                  { 
		    PreMultiplyTransforms(&($2),&($3),&($$));
		    
		    DeleteTransform(&($2));
		    DeleteTransforms(&($3));
		  }
                |
	          {
		    InitTransforms(&($$));
		  }
                ;

transform : transform_type '(' expr ')' opt_inv
            {	      
	      if ($3.id==NULL)
		NewCtTransform($1,$3.value,$5,&($$));
	      else
		{
		  unsigned int id;
		  
		  id=GetVariableID($3.id,&rlp_variables);
		  
		  if (id!=NO_UINT) /*a known variable*/
		    {
		      if ((($1==TR_RX)||($1==TR_RY)||($1==TR_RZ))&&
			  (GetVariableType(GetVariable(id,&rlp_variables))!=VAR_ROT))
			{
			  char s[250];
		      
			  sprintf(s,"Rotations must involve an angular variable %.150s",$3.id);
			  ReadloopsSemanticError(s);
			}
		      if ((($1==TR_TX)||($1==TR_TY)||($1==TR_TZ))&&
			  (GetVariableType(GetVariable(id,&rlp_variables))==VAR_ROT))
			{
			  char s[250];
		      
			  sprintf(s,"Translations must not involve an angular variable %.150s",$3.id);
			  ReadloopsSemanticError(s);
			}
				  
		      NewVarTransform($1,id,$5,&($$));
		    }
		  else
		    {
		      char s[250];
		      
		      sprintf(s,"Unknown variable %.150s",$3.id);
		      ReadloopsSemanticError(s);
		    }
		  free($3.id);
		}
	    }
          | _DHR '(' expr ',' expr ','  expr ','  expr ')' opt_inv
            {
	      unsigned int id;

	      // DH parameters are given as d,t,a,alpha
	      // In this particular case t is variable and the rest are constants
	      
	      if (($3.id!=NULL)||($7.id!=NULL)||($9.id!=NULL))
		ReadloopsSemanticError("DHR d/a/alpha parameters must be constant");
	      if ($5.id==NULL)
		ReadloopsSemanticError("DHR theta parameter must be a variable");
		
	       id=GetVariableID($5.id,&rlp_variables);
	       if (id!=NO_UINT) /*a known variable*/
		 {
		   if (GetVariableType(GetVariable(id,&rlp_variables))!=VAR_ROT)
		     {
		       char s[250];
		       
		       sprintf(s,"DHR transforms must involve an angular variable %.150s",$3.id);
		       ReadloopsSemanticError(s);
		     }
		   NewDHRTransform($3.value,id,$7.value,$9.value,$11,&($$));
		 }
	        else
		  {
		    char s[250];
		    
		    sprintf(s,"Unknown variable %.150s",$3.id);
		    ReadloopsSemanticError(s);
		  }
	       free($5.id);
	    }
          | _TRM '(' expr ','  expr ','  expr ','  expr ';' expr ','  expr ','  expr ','  expr ';' expr ','  expr ','  expr ','  expr ')' opt_inv
            {
	       if (($3.id!=NULL)||($5.id!=NULL)||($7.id!=NULL)||($9.id!=NULL)||
		   ($11.id!=NULL)||($13.id!=NULL)||($15.id!=NULL)||($17.id!=NULL)||
		   ($19.id!=NULL)||($21.id!=NULL)||($23.id!=NULL)||($25.id!=NULL))
		ReadloopsSemanticError("TR can not be defined on variable expresions");
	       
	       NewTransformFromMatrix($3.value,$5.value,$7.value,$9.value,
				      $11.value,$13.value,$15.value,$17.value,
				      $19.value,$21.value,$23.value,$25.value,$27,&($$));
	    }
          | _TRQ '(' expr ','  expr ','  expr ','  expr ',' expr ','  expr ','  expr ')' opt_inv
            {
	       if (($3.id!=NULL)||($5.id!=NULL)||($7.id!=NULL)||
		   ($9.id!=NULL)||($11.id!=NULL)||($13.id!=NULL)||($15.id!=NULL))
		ReadloopsSemanticError("TR can not be defined on variable expresions");
	       
	       NewTransformFromDualQuat(1.0,$3.value,$5.value,$7.value,
					$9.value,$11.value,$13.value,$15.value,$17,&($$));
	    }
	  | _ID
	    {
	      NewIDTransform(&($$));
	    }
          ;

transform_type : _RX
                 {
		   $$=TR_RX;
		 }
               | _RY
                 {
		   $$=TR_RY;
		 }
               | _RZ
                 {
		   $$=TR_RZ;
		 }
               | _TX
                 {
		   $$=TR_TX;
		 }
               | _TY
                 {
		   $$=TR_TY;
		 }
               | _TZ
                 {
		   $$=TR_TZ;
		 }
               ;

opt_inv : _INV
          {
	    $$=TRUE;
	  }
        |
          {
	    $$=FALSE;
	  }
        ;



%%

#include "readcuiksystemtypes.h"

/** \endcond */

void Loops2Cuik(Tparameters *p,char *filename)
{
  Tfilename floops;

  CreateFileName(NULL,filename,NULL,LOOPS_EXT,&floops);
  
  fprintf(stderr,"Creating loops file: %s\n",GetFileFullName(&floops));
  
  Readloopsin=fopen(GetFileFullName(&floops),"r");
  if (Readloopsin) 
    {
      Tfilename fcuik;
      FILE *f;
      
      /*Start the data structures*/
      InitConstants(&rlp_constants);
      InitVariables(&rlp_variables);
      InitLoops(&rlp_loops);
  
      rlp_parameters=p;
  
      /*Reset the lines numbering*/
      RLPline=1;
      
      /*and process the file*/
      Readloopsparse();

      fclose(Readloopsin);

      CreateFileName(NULL,filename,NULL,CUIK_EXT,&fcuik);

      f=fopen(GetFileFullName(&fcuik),"w");
      if (f)
	{
	  unsigned int i,nl,nva,nv;
	  Tvariable *var;
	  char **varNames;

	  
	  fprintf(stderr,"Creating cuik file : %s\n",GetFileFullName(&fcuik));
      
	  nl=NLoops(&rlp_loops);
	  nva=NVariables(&rlp_variables);          //all variables (including offsets)
	  nv=nva-NOffsetVariables(&rlp_variables); //non-offset variables

	  fprintf(f,"[VARS]\n  ");
	  for(i=0;i<nv;i++)
	    {	
	      var=GetVariable(i,&rlp_variables);
	      PrintVariable(f,var);
	      fprintf(f,"\n  ");
	    }

	  NEW(varNames,nva,char*);
	  GetVariableNames(varNames,&rlp_variables);
	  
	  fprintf(f,"\n[EQS]\n  ");
	  for(i=0;i<nl;i++)
	    {
	      PrintTransformsEquations(f,varNames,GetLoop(i,&rlp_loops));
	      fprintf(f,"\n  ");
	    }

	  free(varNames);
	  fclose(f);
	}
      
      DeleteFileName(&fcuik);
      DeleteFileName(&floops);
      
      DeleteVariables(&rlp_variables);
      DeleteLoops(&rlp_loops);
      DeleteConstants(&rlp_constants);
    }
}

/*
 * Reads a file containing the constants definitions, kinematic equations to be solved, 
 * and the range constrains for the variables.
 */
unsigned int ReadLoopsFile(Tparameters *p,char *filename,TCuikSystem **csa)
{
  unsigned int i,no,ns,nv,s,*sID,*vT,nl,nva,nvls,nels;
  Tvariable_set *vars;
  Tvariable *var;
  boolean *ov,*usedVars;
  Tinterval *r;
  double of;
  Tloops simpLoops;
  double *low,*up;
  unsigned int simplify;
  Tloops *loopsS;

  struct timespec t0,t1;
  
  clock_gettime(CLOCK_REALTIME,&t0);
  
  ns=0;
  fprintf(stderr,"Reading loops file: %s\n",filename);
  
  Readloopsin=fopen(filename,"r");
  if (Readloopsin) 
    {
      
      /*Start the data structures*/
      InitConstants(&rlp_constants);
      InitVariables(&rlp_variables);
      InitLoops(&rlp_loops);
  
      rlp_parameters=p;
  
      /*Reset the lines numbering*/
      RLPline=1;
      
      /*and process the file*/
      Readloopsparse();

      fclose(Readloopsin);
      
      clock_gettime(CLOCK_REALTIME,&t1);
      fprintf(stderr,"Parsing time   : %f seconds\n",(t1.tv_sec+t1.tv_nsec*1e-9)-(t0.tv_sec+t0.tv_nsec*1e-9));
      
      /* Generate the memory structure necessary to solve the problem */

      /*
	We perform a simplificitaion without offsets just to find out how many 
	variables and equations will appear in the final system.
	This simplification is latter not used to define sub-problems, which
	depend on the offsets.
       */
      simplify=(unsigned int)GetParameter(CT_SIMPLIFY,p);

      if (simplify>0)
	{
	  SimplifyLoops(simplify,NULL,&simpLoops,&rlp_loops); // just copy if simplify == 0
	  loopsS=&simpLoops;
	}
      else
	loopsS=&rlp_loops;

      //PrintLoopsMatrixVector(stderr,NULL,loopsS); fprintf(stderr,"\n\n\n");
      //exit(0);

      // The total number of variables, even those not used in the loops
      nl=NLoops(loopsS);
      nva=NVariables(&rlp_variables);          //all variables (including offsets)
      nv=nva-NOffsetVariables(&rlp_variables); //non-offset variables
      
      NEW(usedVars,nv,boolean);
      for(i=0;i<nv;i++)
	usedVars[i]=FALSE;
      nels=NEquationsInLoops(loopsS);
      vars=GetVariablesInLoops(loopsS);
      nvls=VariableSetSize(vars);
      for(i=0;i<nvls;i++)
	usedVars[GetVariableN(i,vars)]=TRUE;
      
      no=0;                     // Number of offset variables (among the used vars!)
      NEW(sID,nv,unsigned int); // Indices of the variables affected by the offsets (at most 'nv')
      NEW(ov,nv,boolean);       // TRUE for variables affected by offset
      NEW(vT,nv,unsigned int);  // Variable type

      for(i=0;i<nv;i++)
	{
	  var=GetVariable(i,&rlp_variables);
	  
	  vT[i]=GetVariableType(var);  
	  
	  ov[i]=FALSE;
	  if (vT[i]==VAR_ROT)
	    {
	      r=GetVariableInterval(var);
	      if ((LowerLimit(r)==-M_PI)&&(UpperLimit(r)==+M_PI))
		{
		  if (usedVars[i])
		    {
		      ov[i]=TRUE;
		      sID[no]=i;
		      no++;
		    }
		}
	    }
	}
      
      // The box limits are the same for all subproblems. Set then once     
      of=GetParameter(CT_OFFSET,p);// size of the assymmetry. Can be random too
      
      NEW(low,nv,double);
      NEW(up,nv,double);
      
      for(i=0;i<nv;i++)
	{
	  var=GetVariable(i,&rlp_variables);
	  r=GetVariableInterval(var);

	  if (usedVars[i])
	    {
	      // Offset variables are in the range [-pi,pi]
	      // where this range is interpreted according to
	      // the offset
	      if (ov[i])
		{
		  // of == slight rotation of the covered range
		  if (of==0)
		    {
		      low[i]=-1;
		      up[i] =+1;
		    }
		  else
		    {
		      low[i]=A2T(-M_PI_2+of);
		      up[i] =A2T( M_PI_2+of);
		    }
		}
	      else
		{
		  // For non-offset variables, we interprete the
		  // range according to the variable type
		  if (vT[i]==VAR_ROT)
		    {
		      low[i]=A2T(LowerLimit(r));
		      up[i] =A2T(UpperLimit(r));
		    }
		  else
		    {
		      low[i]=LowerLimit(r);
		      up[i] =UpperLimit(r);
		    }
		}
	    }
	  else
	    {
	      // For non-used variables we just keep the ranges given
	      // in the problem description
	      low[i]=LowerLimit(r);
	      up[i] =UpperLimit(r);
	    }
	}

      // Number of sub-problems
      ns=(1<<no);
      
      NEW(*csa,ns,TCuikSystem);

      #pragma omp parallel for private(s) if (!_DEBUG)
      for(s=0;s<ns;s++)
	{
	  TCuikSystem *cs;
	  double *c;  // corners
	  unsigned int i,j,k,l,n,neql,nvl,nc,*vsl,v;
	  Ttransforms *ts;
	  Tvariable_set *vars;
	  double q[MAX_EQS_LOOP];
	  Tloops reSimpLoops; // the simplified loops for the current offset
	  Tloops *loopsRS;

	  #if (_DEBUG>0)
	    fprintf(stderr,"- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n");
	    fprintf(stderr,"Generating problem: %u\n",s);
	  #endif
	    
	  cs=&((*csa)[s]);

	  /* 
	     The simplification depends on the offsets.
	     Note that the simplification reduces the number of equations and the number of
	     actually used variables. Despite this, we formulate the sub-problems with ALL
	     the variables. The ones not in the simplified equations are just unused variables.
	   */
	  
	  #if (_DEBUG>0)
	    fprintf(stderr,"  Generating offsets\n");
	  #endif
	    
	  NEWZ(cs->offset,nv,signed int); /* space for the offsets (one for variable). Default = No offset */
	  for(i=0;i<no;i++)
	    {
	      if (s & (1<<i))
		cs->offset[sID[i]]=+1;
	      else
		cs->offset[sID[i]]=-1;
	    }
	  
	  #if (_DEBUG>0)
	    fprintf(stderr,"  Re-simplify\n");
	  #endif
	  if (simplify>0)
	    {
	      ReSimplifyLoops(cs->offset,&reSimpLoops,&simpLoops); // nothing if simplify==0
	      loopsRS=&reSimpLoops;
	    }
	  else
	    loopsRS=&rlp_loops;
	   
	  #if (_DEBUG>0)
	    /* Minor sanity check */
	    if (nels!=NEquationsInLoops(loopsRS))
	      Error("Sub-problem simplification has less equations than generic simplification");
	  
	    if (nvls!=NVariablesInLoops(loopsRS))
	      Error("Sub-problem simplification has less variables than generic simplification");
	  #endif
	  
	  /* From this point the system is simplified. Less equations and uses less variables. */

	  cs->ne=NEquationsInLoops(loopsRS); // The simplified number of equations!
	  cs->nv=nv; // All the variables in the problem. Some might be unused.

	  // varType is the same for all sub-problems
	  NEW(cs->varType,cs->nv,unsigned int);
	  memcpy(cs->varType,vT,cs->nv*sizeof(unsigned int));

	  NEW(cs->usedVars,cs->nv,boolean);
	  memcpy(cs->usedVars,usedVars,cs->nv*sizeof(boolean));
	  
	  NEW(cs->nve,cs->ne,unsigned int);
	  NEW(cs->ve,cs->ne,unsigned int*);
	  NEW(cs->nc,cs->ne,unsigned int);
	  k=0;

	  #if (_DEBUG>0)
	    fprintf(stderr,"  Identifying vars in each loop equation\n");
	  #endif
	       
	  for(i=0;i<nl;i++)
	    {
	      ts=GetLoop(i,loopsRS);          // The i-th loop
	      neql=NTransformsEquations(ts);  // Num eqs in this loop
	      vars=GetTransformsVariables(ts);// Variables in this loop
	      nvl=VariableSetSize(vars);      // Num variables in the loop
	      vsl=GetVariables(vars);         // Set of variables in the loop
	      nc=(1<<nvl);
	      
	      // For all the equations in this loop, we have the same variables
	      for(j=0;j<neql;j++)
		{
		  cs->nve[k]=nvl;
		  NEW(cs->ve[k],nvl,unsigned int);
		  memcpy(cs->ve[k],vsl,nvl*sizeof(unsigned int));
		  cs->nc[k]=nc;
		  k++;
		}
	    }
	  
	  #if (_DEBUG>0)
	    fprintf(stderr,"  Defining initial box\n");
	  #endif
	  // The search box for this sub-problem. 
	  NEW(cs->b0,1,TCuikBox);
	  InitCuikBox(low,up,cs->nv,cs->b0);
	   
	  // Space for the function evaluation
	  NEW(cs->f0,cs->ne,double*);
	  for(i=0;i<cs->ne;i++)
	    {
	      NEW(cs->f0[i],cs->nc[i],double);
	    }

	  // Space for the corners. Used to evaluate the equations (zero for non-used variables)
	  NEW(c,cs->nv,double);
	  
	  #if (_DEBUG>0)
	    fprintf(stderr,"  Evaluating loops\n");
	  #endif
	  // We have to evaluate the equations loop-wise
	  l=0;
	  for(i=0;i<nl;i++)
	    {
	      ts=GetLoop(i,loopsRS);          // The i-th loop
	      neql=NTransformsEquations(ts);  // The number of equations in the loop
	      vars=GetTransformsVariables(ts);// The variables in the loop
	      nvl=VariableSetSize(vars);      // The number of variables in the loop
	      vsl=GetVariables(vars);         // The variables in the loop
	      nc=(1<<nvl);                    // Number of corners with the variables
	      
	      #if (_DEBUG>1)
	        fprintf(stderr,"    Loop: %u\n",i);
	        fprintf(stderr,"      Num vars  : %u\n",nvl);
	        fprintf(stderr,"      Num corner: %u\n",nc);
	      #endif
	      // For all corners
	      for(j=0;j<nc;j++)  
		{
		  // Define the corner
		  for(k=0;k<nvl;k++) 
		    {
		      v=vsl[k];
		      if (j & (1<<k)) // First variable is in the lowest bit (right hand one)
			c[v]=up[v];
		      else
			c[v]=low[v];
		    }
		  
		  EvaluateTransforms(c,cs->offset,q,ts);
		    
		  for(n=0;n<neql;n++)
		    cs->f0[l+n][j]=q[n];
		}
	      l+=neql;
	    }

	  // The simplification information. Right now only available from problems
	  // read from loops files!!! (note stored in maf files for backward compatibility)
	  if (simplify==2)
	    {
	      Tvariable_set *rvars;
			  
	      #if (_DEBUG>0)
	        fprintf(stderr,"  Creating simplification information\n");
	      #endif
		
	      cs->nSimp=nl;
	      // Define the simplication information
	      
	      NEW(cs->nIn,cs->nSimp,unsigned int);
	      NEW(cs->nOut,cs->nSimp,unsigned int);
	      NEW(cs->vIn,cs->nSimp,unsigned int*);
	      NEW(cs->vOut,cs->nSimp,unsigned int*);
	      NEW(cs->S,cs->nSimp,double*);

	      for(i=0;i<nl;i++)
		{
		  ts=GetLoop(i,loopsRS);                   // The i -th loop
		  vars=GetTransformsVariables(ts);         // The variables in the loop
		  rvars=GetTransformsRemovedVariables(ts); // The removed variables in the simplification

		  cs->nIn[i]=VariableSetSize(vars);
		  NEW(cs->vIn[i],cs->nIn[i],unsigned int);
		  memcpy(cs->vIn[i],GetVariables(vars),cs->nIn[i]*sizeof(unsigned int));
		  
		  cs->nOut[i]=VariableSetSize(rvars);
		  NEW(cs->vOut[i],cs->nOut[i],unsigned int);
		  memcpy(cs->vOut[i],GetVariables(rvars),cs->nOut[i]*sizeof(unsigned int));
		  
		  GetTransformSimplification(&(cs->S[i]),ts);
		}
	    }
	  else
	    {
	      cs->nSimp=0;
	      cs->nIn=NULL;
	      cs->nOut=NULL;
	      cs->vOut=NULL;
	      cs->S=NULL;
	    }
	  if (simplify>0)
	    DeleteLoops(&reSimpLoops);
	  free(c);
	}
      
      /* Release memory */
      free(usedVars);
      free(ov);
      free(sID);
      free(vT);
      free(low);
      free(up);

      DeleteVariables(&rlp_variables);
      if (simplify>0)
	DeleteLoops(&simpLoops);
      DeleteLoops(&rlp_loops);
      DeleteConstants(&rlp_constants);
    }
  
  return(ns); // number of generated sub-problems
} 

