%{ 
/*
* Lexical analizer for parameter files
*/

/** \cond */

#include <stdlib.h> 
#include <string.h>

#include "defines.h"

#include "readparameters.tab.h"

int ReadParameterslex(void);

extern unsigned int RPNline; /*current line number (declared in the YACC module and updated here)*/

/** \endcond */

%} 
 
%s IN_COMMENT

EMPTY                           [ \t]+ 
RETURN                          \n
DIGIT                           [0-9] 
LETTER                          [a-zA-Z] 
R_NUMBER                        (({DIGIT}+(\.{DIGIT}+)?)|(\.{DIGIT}+))((E|e)[+-]?{DIGIT}+)? 
I_NUMBER                        {DIGIT}+ 
IDENTIFIER                      {LETTER}({DIGIT}|{LETTER}|"_")* 
COMMENT                         \%[^\n]*
COMMENT2                        \#[^{\n|\{}]* 
COMMENT3                        \/\/[^{\n|\r\n|\{}]*
 
%%

{COMMENT}                       { }   
{COMMENT2}                      { }  
{COMMENT3}                      { }  

<INITIAL>{
  "/*"              BEGIN(IN_COMMENT);
}
<IN_COMMENT>{
  "*/"      BEGIN(INITIAL);
  [^*\n]+   // eat comment in chunks
  "*"       // eat the lone star
  \n        RPNline++;
}

{EMPTY}                         { } 

{RETURN}                        { RPNline++; } 

TRUE                            { return(_TRUE);}
FALSE                           { return(_FALSE);}

ON                              { return(_TRUE);}
OFF                             { return(_FALSE);}

OR                              { return(_OR); }
AND                             { return(_AND); }
NOT                             { return(_NOT); }

'>='                            { return(_GEQ); }
'<='                            { return(_LEQ); }
'=='                            { return(_EQU); }

PI                              { return(_PI);}
COS                             { return(_COS); }
SIN                             { return(_SIN); }
TAN                             { return(_TAN); }
EXP                             { return(_EXP); }
LOG                             { return(_LOG); }
SQRT                            { return(_SQRT); }

INF                             { return(_INF); }

EPSILON                         { return(_EPSILON); }

RHO                             { return(_RHO); }

SIGMA                           { return(_SIGMA); }

REDUCTION                       { return(_REDUCTION); }
TRAPEZOID                       { return(_TRAPEZOID); }
SIMPLEX                         { return(_SIMPLEX); }

MIRANDA                         { return(_MIRANDA); }
MIRANDA_NEWTON                  { return(_NEWTON); }
MIRANDA_NEWTON_REPLACE          { return(_NEWTON_REPLACE); }

CLEAN                           { return(_CLEAN); }

VERIFY                          { return(_VERIFY); }

SIMPLIFY                        { return(_SIMPLIFY); }

ROTATE_LOOPS                    { return(_ROTATE_LOOPS); }

ALL                             { return(_ALL); }
ALL_VARS                        { return(_ALL_VARS); }

SAVE_ANGLES                     { return(_SAVE_ANGLES); }

OFFSET                          { return(_OFFSET); }

{I_NUMBER}                      {
				  char string_tmp[100];
				  
				  memcpy(string_tmp,ReadParameterstext,(unsigned int)ReadParametersleng);
				  string_tmp[ReadParametersleng]=0;
				  ReadParameterslval.realValue=(double)atoi(string_tmp);
				  return(_INTEGER); 
				} 
 
{R_NUMBER}                      { 
				  char string_tmp[100];
				  
				  memcpy(string_tmp,ReadParameterstext,(unsigned int)ReadParametersleng);
				  string_tmp[ReadParametersleng]=0;
				  ReadParameterslval.realValue=atof(string_tmp);
				  
				  return(_REAL);
				}
 
.                               { return(ReadParameterstext[0]); }
 
%%

/**
  \brief Function called at the end of the parsing process.

  This function is automatically called by the parser at the end of a file.
  If 1 is returned the lexical analysis is stopped.
 
  \return 1 to stop the parsing. 
*/
int ReadParameterswrap()
{
  return(1);
}
   
