%{ 
/** \cond */

#include "parameters.h"
#include "error.h"
#include "boolean.h"
#include "defines.h"

#include <stdlib.h> 
#include <stdio.h>
#include <math.h>

  Tparameters *rp_ps; /*global pointer used from inside the parser to refer to the current set of
		       parameter being read*/

  /*Lex and Yacc variables*/
  extern FILE *ReadParametersin; 

  /*Lex and Yacc functions*/
  int ReadParameterslex(void); 

  /*Our own variables*/
  extern unsigned int RPNline; /*line number currently processed (incremented by the LEX processor)*/

%}
%union 
{ 
  double realValue;
  unsigned int intValue;
} 
 
%start parameters

%token _TRUE _FALSE _OR _AND _NOT _GEQ _LEQ _EQU _PI _COS _SIN _TAN _EXP _LOG _SQRT _INF _EPSILON  _SIGMA _RHO _REDUCTION _TRAPEZOID _SIMPLEX _MIRANDA _NEWTON _NEWTON_REPLACE _CLEAN _VERIFY _SIMPLIFY _ROTATE_LOOPS _ALL _ALL_VARS _SAVE_ANGLES _OFFSET 

%token <realValue> _INTEGER
%token <realValue> _REAL


%type <intValue> clean_method
%type <intValue> verify_method
%type <intValue> reduction_method 
%type <intValue> rotate_method 
%type <realValue> ct
%type <realValue> expr
%type <intValue> boolean_value

%left _MIN_PRECEDENCE
%left '<' '>' _LEQ _GEQ _EQU
%left '+' '-' _OR
%left '*' '/' _AND
%left '^' _NOT
%right _MAX_PRECEDENCE 

%% 

parameters : parameter parameters
           |
	   ;

parameter : _EPSILON '=' expr
            {
	      // 0
	      if ($3<0.0)
		Error("Parameter EPSILON must be greater than (or equal to) 0.0");
	      SetParameter(CT_EPSILON,"EPSILON",$3,rp_ps);
            }
          | _RHO '=' expr
            {
	      // 1
	      if (($3>0.0)&&($3<1.0))
		SetParameter(CT_RHO,"RHO",$3,rp_ps);
	      else
		Error("Parameter RHO must in the interval (0,1)");
	    }
          | _SIGMA '=' expr
            {
	      // 2
	      if ($3<0.0)
		Error("Parameter SIGMA  must be greater than (or equal to) 0.0");
	      SetParameter(CT_SIGMA,"SIGMA",$3,rp_ps);
            }
          | _REDUCTION '=' reduction_method
            {
	      // 3
	      SetParameter(CT_REDUCTION,"REDUCTION",$3,rp_ps);
            }
          | _CLEAN '=' clean_method
            {
	      // 4
	      SetParameter(CT_CLEAN,"CLEAN",$3,rp_ps);
            }
          | _VERIFY '=' verify_method
            {
	      // 5
	      SetParameter(CT_VERIFY,"VERIFY",$3,rp_ps);
            }
          | _SIMPLIFY '=' _INTEGER
	    {
	      // 6
	      SetParameter(CT_SIMPLIFY,"SIMPLIFY",$3,rp_ps);
	    }
          | _ROTATE_LOOPS '=' rotate_method
	    {
	      // 7
	      SetParameter(CT_ROTATE_LOOPS,"ROTATE_LOOPS",$3,rp_ps);
	    }
          | _SAVE_ANGLES '=' boolean_value
            {
	      // 8
	      SetParameter(CT_SAVE_ANGLES,"SAVE_ANGLES",$3,rp_ps);
            }
          | _OFFSET '=' expr
	    {
	      // 9
	      SetParameter(CT_OFFSET,"OFFSET",$3,rp_ps);
	    }
          ;

clean_method : _FALSE
               {
		 $$=NO_CLEAN;
	       }
             | _SIMPLEX
               {
		 $$=CLEAN_SIMPLEX;
	       }
             ;

verify_method : _MIRANDA
                {
		  $$=VERIFY_MIRANDA;
		}
              | _NEWTON
	        {
		  $$=VERIFY_NEWTON;
		}
              | _NEWTON_REPLACE
	        {
		  $$=VERIFY_NEWTON_REPLACE;
		}
              ;

rotate_method : _FALSE
                {
		  $$=0;
		}
              | _ALL
                {
		  $$=1;
		}
              | _ALL_VARS
                {
		  $$=2;
		}
              ;

reduction_method : _TRAPEZOID 
                   {
		     $$=RED_TRAPEZOID;
		   }
                 | _SIMPLEX
                   {
		     $$=RED_SIMPLEX;
		   }
                 ;

ct : _EPSILON
     {
       // 0
       if (ParameterSet(CT_EPSILON,rp_ps))
	 $$=GetParameter(CT_EPSILON,rp_ps);
       else
	 Error("Undefined parameter in expression (EPSILON)");
     }
  | _RHO
    {
      // 2
      if (ParameterSet(CT_RHO,rp_ps))
	$$=GetParameter(CT_RHO,rp_ps);
      else
	Error("Undefined parameter in expression (RHO)");
    }
  | _SIGMA
    {
      // 3
      if (ParameterSet(CT_SIGMA,rp_ps))
	$$=GetParameter(CT_SIGMA,rp_ps);
      else
	Error("Undefined parameter in expression (SIGMA)");
    }
  | _OFFSET
    {
      // 3
      if (ParameterSet(CT_OFFSET,rp_ps))
	$$=GetParameter(CT_OFFSET,rp_ps);
      else
	Error("Undefined parameter in expression (OFFSET)");
    }
  ;

expr : _INTEGER
       {
	 $$=$1;
       }
     | _REAL
       {
	 $$=$1;
       }
     | _PI
       {
	 $$=M_PI;
       }
     | _INF
       {
	 $$=INF;
       }
     | boolean_value
       {
	 $$=(double)$1;
       }
     | ct
       {
	 $$=$1;
       }
     | '+' expr                 %prec _MAX_PRECEDENCE
       {
	 $$=$2;
       }
     | '-' expr                 %prec _MAX_PRECEDENCE
       {
	 $$=-$2;
       }
     | '(' expr ')'
       {
	 $$=$2;
       }
     | '(' expr '?' expr ':' expr ')'
       { 
	 if ($2>0)
	   $$=$4;
	 else
	   $$=$6;
       }
     | expr '+' expr
       {
	 $$=$1+$3;
       }
     | expr '-' expr
       { 
	 $$=$1-$3;
       }
     | expr '*' expr
       {
	 $$=$1*$3;
       }
     | expr '^' expr
       {
	 $$=pow($1,$3);
       }
     | expr '/' expr
       {
	 $$=$1/$3;
       }
     | _COS '(' expr ')'
       {
	 $$=cos($3);
       }
     | _SIN '(' expr ')'
       {
	 $$=sin($3);
       }
     | _TAN '(' expr ')'
       {
	 $$=tan($3);
       }
     | _EXP '(' expr ')'
       {
	 $$=exp($3);
       }
     | _LOG '(' expr ')'
       {
	 $$=log($3);
       }
     | _SQRT '(' expr ')'
       {
	 $$=sqrt($3);
       }
     | _NOT expr
       {
	 if ((fabs($2-floor($2))>1e-10)||($2<0)||($2>1))
	   Error("Boolean operation on a non-logical expresion");
	 $$=!(($2)>0);
       }
     | expr _AND expr
       {
	 if ((fabs($1-floor($1))>1e-10)||($1<0)||($1>1))
	   Error("Boolean operation on a non-logical expresion");	 
	 if ((fabs($3-floor($3))>1e-10)||($3<0)||($3>1))
	   Error("Boolean operation on a non-logical expresion");
	 
	 $$=((($1)>0)&&(($3)>0));
       }
     | expr _OR expr
       {
	 if ((fabs($1-floor($1))>1e-10)||($1<0)||($1>1))
	   Error("Boolean operation on a non-logical expresion");	 
	 if ((fabs($3-floor($3))>1e-10)||($3<0)||($3>1))
	   Error("Boolean operation on a non-logical expresion");
	 
	 $$=((($1)>0)||(($3)>0));
       }
     | expr '<' expr
       { 
	 $$=(($1)<($3));
       }
     | expr '>' expr
       { 
	 $$=(($1)>($3));
       }
     | expr _LEQ expr
       { 
	 $$=(($1)<=($3));
       }
     | expr _GEQ expr
       { 
	 $$=(($1)>=($3));
       }
     | expr _EQU expr
       { 
	 $$=(($1)==($3));
       }
     ;

boolean_value : _TRUE
                {
	          $$=1;
                }
              | _FALSE
                {
	          $$=0;
                }
              ;

%%
/** \endcond */
boolean ReadParameters(char *file,Tparameters *p)
{
  unsigned int i;
  boolean fileExists;

  ReadParametersin=fopen(file,"r");
  if (!ReadParametersin) 
    fileExists=FALSE;
  else
    {
      fileExists=TRUE;
    
      /*Set up the global pointer to make the parameter structure under initialization
	to be accesible form everywhere in the parser*/
      rp_ps=p;
  
      /*Reset the lines numbering*/
      RPNline=1;

      /*and process the file*/
      ReadParametersparse();

      for(i=0;i<NPARAMETERS;i++)
	{
	  if ((*p)[i].name==NULL)
	    {
	      char ErrorText[200];
	      
	      sprintf(ErrorText,"Parameter number %u is undefined (see sources/parameters.h to see parameter numbering)",i);
	      Error(ErrorText);
	    }
	}

      if (GetParameter(CT_EPSILON,p)>GetParameter(CT_SIGMA,p))
	Error("EPSILON can not be larger than SIGMA");

      fclose(ReadParametersin);
    }

  return(fileExists);
} 
