#include "variable.h"

#include "error.h"
#include "defines.h"

#include <stdlib.h>
#include <string.h>


/** 
   \file variable.c

   \brief Implementation of the functions operating on box Tvariable. 

   Implementation of the functions operating on box Tvariable. 

   \sa Tvariable, variable.h.
*/

void NewVariable(char *name,unsigned int varType,Tvariable *v)
{
  NEW(v->name,strlen(name)+1,char);
  strcpy(v->name,name);

  if (varType==VAR_OFFSET)
    Error("Use NewOffsetVariable to define OFFSET variables");
  v->varType=varType;
  v->sv=NO_UINT;
  
  NewInterval(-INF,INF,&(v->is)); /*default range for the variable*/
}

void NewOffsetVariable(char *name,unsigned int sv,Tvariable *v)
{
  NEW(v->name,strlen(name)+1,char);
  strcpy(v->name,name);

  v->varType=VAR_OFFSET;
  if (sv==NO_UINT)
    Error("Shifted variable is missing");
  v->sv=sv;
  
  NewInterval(-1.0,1.0,&(v->is)); /*default range for the variable*/
}

void CopyVariable(Tvariable *v_dst,Tvariable *v_src)
{
  NEW(v_dst->name,strlen(v_src->name)+1,char);
  strcpy(v_dst->name,v_src->name);

  v_dst->varType=v_src->varType;
  v_dst->sv=v_src->sv;
  
  CopyInterval(&(v_dst->is),&(v_src->is));
}

char *GetVariableName(Tvariable *v)
{
  return(v->name);
}

unsigned int GetVariableType(Tvariable *v)
{
  return(v->varType);
}

boolean IsOffsetVariable(Tvariable *v)
{
  return(v->varType==VAR_OFFSET);
}

unsigned int ShiftedVariable(Tvariable *v)
{
  return(v->sv);
}

void SetVariableInterval(Tinterval *i,Tvariable *v)
{
  if (v->varType==VAR_OFFSET)
    Error("The range for offset variables is predefined");
  
  CopyInterval(&(v->is),i);
}

Tinterval *GetVariableInterval(Tvariable *v)
{
  return(&(v->is));
}

void PrintVariable(FILE *f,Tvariable *v)
{
  PRINT_VARIABLE_NAME(f,v->name);
  if (v->varType!=VAR_OFFSET)
    {
      if (v->varType==VAR_ROT)
	fprintf(f," ~ ");
      else
	fprintf(f," : ");
      PrintInterval(f,&(v->is));
    }
}

void DeleteVariable(Tvariable *v)
{
  free(v->name);
}
